@@ -5,6 +5,8 @@ import hashlib
 import hmac
 import logging
 import re
+import time
+import six
 from typing import Dict, Optional, Union, List
 
 import requests
@@ -14,7 +16,7 @@ from requests_oauthlib.compliance_fixes.facebook import facebook_compliance_fix
 
 from pyfacebook.error import PyFacebookException, ErrorMessage, ErrorCode
 from pyfacebook.models import AccessToken, AuthAccessToken
-from pyfacebook.ratelimit import RateLimit
+from pyfacebook.ratelimit import RateLimit, PercentSecond
 
 
 class BaseApi(object):
@@ -36,6 +38,7 @@ class BaseApi(object):
                  version=None,  # type: Optional[str]
                  timeout=None,  # type: Optional[int]
                  sleep_on_rate_limit=False,  # type: bool
+                 sleep_seconds_mapping=None,  # type: Dict[int, int]
                  proxies=None,  # type: Optional[dict]
                  debug_http=False  # type: bool
                  ):
@@ -50,6 +53,7 @@ class BaseApi(object):
         :param version: The version for the graph api.
         :param timeout: Request time out
         :param sleep_on_rate_limit: Use this will sleep between two request.
+        :param sleep_seconds_mapping: Mapping for percent to sleep.
         :param proxies: Your proxies config.
         :param debug_http: Set to True to enable debug output from urllib when performing
         any HTTP requests.  Defaults to False.
@@ -61,7 +65,7 @@ class BaseApi(object):
         self.base_url = self.GRAPH_URL
         self.proxies = proxies
         self.session = requests.Session()
-        self.sleep_on_rate_limit = sleep_on_rate_limit  # TODO
+        self.sleep_on_rate_limit = sleep_on_rate_limit
         self.instagram_business_id = None
         self._debug_http = debug_http
         self.authorization_url = self.DEFAULT_AUTHORIZATION_URL
@@ -94,6 +98,8 @@ class BaseApi(object):
                     message="Version string is invalid for {0}. You can provide with like: 5.0 or v5.0".format(version),
                 ))
 
+        self.sleep_seconds_mapping = self._build_sleep_seconds_resource(sleep_seconds_mapping)
+
         if long_term_token:
             self._access_token = long_term_token
         elif short_token and all([self.app_id, self.app_secret]):
@@ -127,6 +133,19 @@ class BaseApi(object):
             requests_log.setLevel(logging.DEBUG)
             requests_log.propagate = True
 
+    @staticmethod
+    def _build_sleep_seconds_resource(sleep_seconds_mapping):
+        # type: (Optional[Dict]) -> Optional[List[PercentSecond]]
+        """
+        Sort and convert data
+        :param sleep_seconds_mapping: mapping for sleep.
+        :return:
+        """
+        if sleep_seconds_mapping is None:
+            return None
+        mapping_list = [PercentSecond(percent=p, seconds=s) for p, s in six.iteritems(sleep_seconds_mapping)]
+        return sorted(mapping_list, key=lambda ps: ps.percent)
+
     @staticmethod
     def _generate_secret_proof(secret, access_token):
         # type: (str, str) -> Optional[str]
@@ -186,6 +205,9 @@ class BaseApi(object):
             raise PyFacebookException(ErrorMessage(code=ErrorCode.HTTP_ERROR, message=e.args[0]))
         headers = response.headers
         self.rate_limit.set_limit(headers)
+        if self.sleep_on_rate_limit:
+            sleep_seconds = self.rate_limit.get_sleep_seconds(sleep_data=self.sleep_seconds_mapping)
+            time.sleep(sleep_seconds)
         return response
 
     def _parse_response(self, response):
